
def pickle_dump(obj,fp):
    import pickle
    with open(fp,'wb') as f:
        pickle.dump(obj,f)
def pickle_load(fp):
    import pickle
    with open(fp,'rb') as f:
        return pickle.load(f)
def json_load(f,encoding='utf-8',*args,**kwargs):
    import json
    with open(f,'r',encoding=encoding) as fp:
        return json.load(fp,*args,**kwargs)
def json_dump(obj,fp,encoding='utf-8',ensure_ascii=False,*args,**kwargs):
    import json
    with open(fp,'w',encoding=encoding) as f:
        json.dump(obj,f,ensure_ascii=ensure_ascii,*args,**kwargs)
def read_txt(path,encoding='utf-8'):
    with open(path,'r',encoding=encoding) as f:
        return f.read()
def write_txt(txt,path,encoding='utf-8'):
    with open(path,'w',encoding=encoding) as f:
        f.write(txt)

def yaml_load(f,encoding='utf-8',*args,**kwargs):
    import yaml
    with open(f,'r',encoding=encoding) as fp:
        return yaml.load(fp,*args,**kwargs)
def yaml_dump(obj,fp,encoding='utf-8',allow_unicode=True,**kwargs):
    import yaml
    with open(fp,'w',encoding=encoding) as f:
        yaml.safe_dump(obj,f,allow_unicode=allow_unicode,**kwargs)


import os
USER_HOME=os.path.expanduser('~')
class AppDataManager:
    def __init__(self,cache_dirname):
        self.root = os.path.join(USER_HOME, cache_dirname)
        self.tmp_dir=os.path.join(self.root,'tmp')
        self.data_dir=os.path.join(self.root,'data')
        self.init()
    def make_dirs(self,dirs):
        for d in dirs:
            if not os.path.exists(d):
                os.makedirs(d)
    def init(self):
        self.make_dirs([self.root,self.tmp_dir,self.data_dir])
    def tempfile(self,filename):
        return os.path.join(self.tmp_dir,filename)
app_data_manager=AppDataManager('.pkman')




import os
import subprocess
import sys
import pkg_resources
from pkg_resources import DistributionNotFound, VersionConflict

from .utils import CmdUtil,IoUtil,StringUtil,PathUtil,FireUtil
from .config import PACKAGE_CONFIG_FILENAME,PACKAGE_CONFIG_FILENAME_YML

class Package:
    def __init__(self, path):
        self.path = os.path.abspath(path)
        self.plugins=[]
        self.add_plugins([
            ShellCommandRunner(),
            PkxCommandRunner(),
            QuickCommandRunner(),
            EntryFileRunner(),
            FireFileRunner(),
            PkmanCommandRunner(),
        ])
    def add_plugins(self,plugins):
        self.plugins.extend(plugins)
    def join_path(self, path):
        return os.path.join(self.path,path)
    def load_pkg_info(self):
        config_path = self.join_path(PACKAGE_CONFIG_FILENAME)
        config_path_yml = self.join_path(PACKAGE_CONFIG_FILENAME_YML)
        if os.path.exists(config_path):
            return IoUtil.json_load(config_path)
        elif os.path.exists(config_path_yml):
            return IoUtil.yaml_load(config_path_yml)
        else:
            return None
    def install_packages_in_dist(self):
        dist_path=os.path.join(self.path,"dist")
        fs=os.listdir(dist_path)
        for f in fs:
            if f.endswith(".whl"):
                subprocess.call([sys.executable,"-m","pip","install","dist/"+f])
    def cmd_in_path(self,command):
        return CmdUtil.run_multiple_commands([command],cwd=self.path)
    def install_dependencies(self):
        cfg = self.load_pkg_info()
        if cfg:
            dependencies = cfg['dependencies']
            try:
                pkg_resources.require(dependencies)
            except DistributionNotFound:
                for pkg in dependencies:
                    self.cmd_in_path([sys.executable,'-m','pip','install',pkg])
            except VersionConflict:
                raise VersionConflict
    def run(self, args):
        for plugin in self.plugins:
            res=plugin.use(self,args)
            if plugin.should_stop:
                return res
    def error(self,msg):
        raise Exception('Error: %s'%(msg))
    def no_config_file_error(self):
        self.error('No config file found.')
    def gitsetup(self,branch):
        cfg=self.load_pkg_info()
        if cfg is None:
            self.no_config_file_error()
        else:
            IoUtil.write_txt("\n".join([
                ".idea",'.vscode',".code",".node_modules","local","build","dist",cfg['name']+".egg-info"
            ]),self.join_path('.gitignore'))
            self.cmd_in_path('pkx gitops init %s %s' % (cfg['repository']['url'], branch))
class Plugin:
    def __init__(self):
        self.should_stop=False
    def use(self,pkg:Package,args):
        pass
class EntryFileRunner(Plugin):
    def use(self,pkg:Package,args):
        path = pkg.path
        entry_filepath = os.path.join(path, 'entry.py')
        if os.path.exists(entry_filepath):
            self.should_stop=True
            return CmdUtil.run_file(entry_filepath, args)
        self.should_stop=False
class FireFileRunner(Plugin):
    def use(self,pkg:Package,args):
        path = pkg.path
        fire_filepath = os.path.join(path, 'firex.py')
        if os.path.exists(fire_filepath):
            self.should_stop=True
            return CmdUtil.run_file(fire_filepath, args,fire=True)
        self.should_stop=False
class QuickCommandRunner(Plugin):
    def use(self,pkg:Package,args):
        if len(args)==1 and (args[0] in ['ls','dir']):
            self.should_stop = True
            if os.name=='nt':
                return pkg.cmd_in_path('dir')
            else:
                return pkg.cmd_in_path('ls')
        else:
            self.should_stop=False
class PkmanCommandRunner(Plugin):
    def use(self,pkg:Package,args):
        path = pkg.path
        self.should_stop = True
        return CmdUtil.run_command(['pkman', *args],cwd=path)

class PkxCommandRunner(Plugin):
    def use(self,pkg:Package,args):
        path = pkg.path
        if len(args)>=1 and args[0]=='-x':
            self.should_stop = True
            return CmdUtil.run_command(['pkx', *args[1:]],cwd=path)
        else:
            self.should_stop=False

class ShellCommandRunner(Plugin):
    def use(self,pkg:Package,args):
        path = pkg.path
        if len(args)>=1 and args[0]=='-cmd':
            self.should_stop = True
            return CmdUtil.run_command([*args[1:]],cwd=path)
        else:
            self.should_stop=False




import json
import os
import subprocess
import sys

import setuptools
from . import IoUtil,StringUtil
from .AppDataManager import app_data_manager


def setup_package(name, version, author, author_email, description, url, dependencies, entry_points, package_data, python_requires, classifiers, package_dir=None,
                  cmd=None, **kwargs):
    if cmd is None:
        cmd = ["sdist", "bdist_wheel"]
    if not package_dir:
        package_dir={name:name}
    elif isinstance(package_dir,str):
        package_dir={package_dir:package_dir}

    packages = setuptools.find_packages(exclude=['local', 'local.*'])
    files=os.listdir('.')
    long_description="No long_description",
    long_description_content_type="text/plain"
    for f in files:
        if f.lower()=='readme.md':
            with open(f,'r',encoding='utf-8') as fp:
                long_description=fp.read()
                long_description_content_type="text/markdown"
        elif f.lower()=='readme.txt':
            with open(f,'r',encoding='utf-8') as fp:
                long_description=fp.read()
                long_description_content_type="text/plain"
    cfg = dict(
        # executable=True,
        name=name,  # Replace with your own username
        version=version,
        author=author,
        author_email=author_email,
        description=description,
        long_description=long_description,
        long_description_content_type=long_description_content_type,
        url=url,
        packages=packages,
        package_dir=package_dir,
        install_requires=dependencies,
        include_package_data=True,
        entry_points=entry_points,
        package_data=package_data,
        classifiers=classifiers,
        python_requires=python_requires,
    )
    # cfg.update(**kwargs)
    jsonfile=app_data_manager.tempfile("setup_config_"+StringUtil.hash_text(json.dumps(cfg))[:10]+".json")
    setupfile=app_data_manager.tempfile("setup_"+StringUtil.hash_text(json.dumps(cfg))[:10]+".py")
    IoUtil.json_dump(cfg,jsonfile)
    setuppy_content='''import setuptools
import json
with open(r"%s","r",encoding="utf-8") as f:
    cfg=json.load(f)
setuptools.setup(**cfg)'''%(jsonfile)
    IoUtil.write_txt(setuppy_content,setupfile)
    try:
        subprocess.check_call([sys.executable, setupfile, *cmd], shell=True)
    except Exception as e:
        print(e)
    os.remove(jsonfile)
    os.remove(setupfile)







import subprocess
import sys
import os
from .AppDataManager import AppDataManager
from . import StringUtil,IoUtil,PathUtil,FireUtil
app_data_manager=AppDataManager('.pkman/tmp_code')

def run_command(args,check=False,cwd='.'):
    cmd=" ".join(args) if isinstance(args,(list,tuple)) else args
    if check:
        return subprocess.check_call(cmd, shell=True,cwd=cwd)
    else:
        return subprocess.call(cmd, shell=True,cwd=cwd)
def run_multiple_commands(args_list,check=False,cwd='.'):
    if os.name=='nt':
        return run_command(" & ".join([" ".join(args) if isinstance(args ,list) else args for args in args_list]),check=check,cwd=cwd)
    else:
        return run_command(" && ".join([" ".join(args) if isinstance(args ,list) else args for args in args_list]),check=check,cwd=cwd)
def run_batch_code(code):
    filename=StringUtil.hash_text(code)[:10]+'.bat'
    file_path=app_data_manager.tempfile(filename)
    IoUtil.write_txt(code,file_path)
    res=os.system(file_path)
    os.remove(file_path)
    return res
def run_python_script(path, args, extend_path=None):
    if extend_path is None:
        extend_path = []
    cmd=[sys.executable,path,*args]
    if extend_path:
        if os.name == 'nt':
            code = 'set PYTHONPATH=%s' % (';'.join(extend_path)) + ' & ' + ' '.join(cmd)
            return run_command([code])
        else:
            code='export PYTHONPATH=%s'%(':'.join(extend_path))+' && '+ ' '.join(cmd)
            return run_command([code])
    else:
        return run_command(cmd)
def gen_fire_argv(*args, **kwargs):
    argv = []
    for arg in args:
        argv.append(str(arg))
    for k, v in kwargs.items():
        argv.append('--%s=%s' % (k, v))
    return argv


def run_file(path, args, fire=False, extend_path=None):
    if extend_path is None:
        extend_path = []
    def run_py(path):
        if os.path.exists(path):
            if fire:
                FireUtil.inject_fire(path)
                try:
                    run_python_script(path, args, extend_path=extend_path)
                except Exception as e:
                    print('Error :',e)
                FireUtil.remove_fire(path)
            else:
                run_python_script(path, args, extend_path=extend_path)
        else:
            raise FileNotFoundError(path)
    if path.endswith('.py'):
        run_py(path)
    else:
        run_py(path+'.py')


import json
import os.path

from .utils import IoUtil,OsUtil
from .config import PACKAGE_CONFIG_FILENAME
from .utils.AppDataManager import AppDataManager
from .utils import CmdUtil,IoUtil,StringUtil,PathUtil,FireUtil
from .utils.AppDataManager import app_data_manager

def init_config_file(root_path):

    # if not OsUtil.is_empty_dir(root_path):
    #     raise Exception('Not an empty directory:',root_path)
    def input_field(description, default, keep_empty):
        res = input('%s: (%s)' % (description, default)).strip() if default is not None else input(
            '%s: ' % (description)).strip()
        if res:
            return res
        elif default is not None:
            return default
        elif keep_empty:
            return ""
        else:
            return default

    def field(description, default=None, keep_empty=False):
        return dict(description=description, default=default, keep_empty=keep_empty)

    form = dict(
        name=field(description="package name", default=os.path.basename(os.path.abspath(os.getcwd()))),
        package_dir=field(description="package directory name", default=os.path.basename(os.path.abspath(os.getcwd())).replace("-","_")),
        version=field(description="version", default="0.0.0.1"),
        description=field(description="description", keep_empty=True),
        # entry_point=field(description="entry point", default="main.py"),
        test_command=field(description="test command"),
        git_repository=field(description="git repository"),
        keywords=field(description="keywords (separated with ',')"),
        author=field(description="author", keep_empty=True),
        license=field(description="license", default='MIT'),
        python_requires=field(description="required python version", default='>=3.0'),
    )

    def fill_form(form):
        result = {}
        for k, f in form.items():
            v = input_field(f['description'], f['default'], f["keep_empty"])
            if v is not None:
                result[k] = v
        return result

    info = fill_form(form)
    if "test_command" in info.keys():
        test_command = info.pop("test_command")
    else:
        test_command = 'echo "Error: no test specified"'
    info["scripts"] = {"test": test_command}

    if "git_repository" in info.keys():
        repository = {
            "type": "git",
            "url": info.pop("git_repository")
        }
        info["repository"]=repository
    if "keywords" in info and info["keywords"]:
        info["keywords"]=[x.strip() for x in info["keywords"].split(',')]
    pkg_name=info['name']
    package_dir=info['package_dir']
    extra_fields = dict(
        author_email="",
        url="",
        dependencies=[],
        package_dir=package_dir,
        entry_points={
            'console_scripts': [
                '%s = %s.clitools.cli:main'%(pkg_name,package_dir)
            ]
        },
        include_package_data=True,
        package_data={
            package_dir: [
                'data/*', 'data/*/*', 'data/*/*/*', 'data/*/*/*/*', 'data/*/*/*/*/*', 'data/*/*/*/*/*/*',
                'data/*/*/*/*/*/*/*',
            ]
        },
        classifiers=[
            "Programming Language :: Python :: 3",
            "License :: OSI Approved :: MIT License",
            "Operating System :: OS Independent",
        ],
    )
    info.update(extra_fields)
    pkg_config_path=os.path.join(root_path,PACKAGE_CONFIG_FILENAME)
    IoUtil.json_dump(info,pkg_config_path,indent=2)
    print(IoUtil.read_txt(pkg_config_path))
    return info
def init_readme(pkg_name,description,root_path):
    readme_path=os.path.join(root_path,"README.md")
    readme_content='''# %s\n---\n%s'''%(pkg_name,description)
    IoUtil.write_txt(readme_content,readme_path)
def init(root_path=None):
    if not root_path:
        root_path=os.getcwd()
    info=init_config_file(root_path)
    init_readme(info['name'],info['description'],root_path)

def up_version(root_path=None):
    if not root_path:
        root_path=os.getcwd()
    config_file_path=os.path.join(root_path,PACKAGE_CONFIG_FILENAME)
    cfg=IoUtil.json_load(config_file_path)
    def version_grow(version, index=-1, limit=None):
        if limit is None:
            limit = [9, 9, 9, 9, 9, 9, 9]
        def to_array(v):
            return [int(x) for x in v.split('.')]
        def to_string(v):
            return '.'.join([str(x) for x in v])
        version=to_array(version)
        if index == -1: index = len(version) - 1
        version[index] += 1
        if version[index] > limit[len(version) - index - 1]:
            version[index] = 0
            if index == 0:
                raise Exception('Max version number reached. %s' % (version))
            return version_grow(to_string(version), index - 1,limit)
        else:
            return to_string(version)
    cfg['version']=version_grow(cfg['version'])
    IoUtil.json_dump(cfg,config_file_path,indent=2)



